@@ -1,4 +1,4 @@
-.build_design_matrix <- function(pheno=NULL,  indxNA=NULL, fformula=NULL, quiet=TRUE  )
+.build_design_matrix <- function(pheno=NULL,   fformula=NULL, quiet=TRUE  )
 {
    ## internal function: use only in AM function and SummaryAM  function
    ## build design matrix given character vector fformula of column names
@@ -6,30 +6,14 @@
    ## assign model matrix X
    if(is.null(fformula))
    {  ## trait + intercept being fitted only
-      if(length(indxNA) > 0){
-         Xmat <- matrix(data=1, nrow=nrow(pheno[-indxNA,]), ncol=1)
-
-      } else {
-        Xmat <- matrix(data=1, nrow=nrow(pheno), ncol=1)
-      }
+      Xmat <- matrix(data=1, nrow=nrow(pheno), ncol=1)
       colnames(Xmat) <- "(Intercept)"  ## to be consistent with model.matrix
    } else {
       ## trait + fixed effects being fitted. 
-     if(length(indxNA)==0 | is.null(indxNA) )
-     {
-        Xmat <- model.matrix(fformula, data=pheno)
-     }  else {
-        # there is an issue with creating Xmat when it includes
-        # factors that have some of their levels removed. 
-        ph <- pheno[-indxNA,]
-        mat <- get_all_vars(formula=fformula, data=ph)
-        for(ii in names(mat)){
-           if(is.factor(ph[,ii])){
-              ph[,ii] <- as.factor(as.character(ph[,ii]))
-           }
-        }  ## for    
-        Xmat <- model.matrix(fformula, data=ph)
-     } ## if else (length(indxNA)==0)
+      current.na.action <- options('na.action')[[1]]
+      options(na.action='na.pass')   # this leaves NA's in the model matrix, preserving its correct dim
+      Xmat <- model.matrix(fformula, data=pheno)
+      options(na.action=current.na.action)
    }
 
  if (!quiet ){
