@@ -1,12 +1,14 @@
- .calc_extBIC <- function(trait=NULL, currentX=NULL, MMt=NULL,  geno=NULL, Zmat=NULL, numberSNPselected=0, quiet=TRUE, gamma=NULL)
+ .calc_extBIC <- function( ML=0 , trait=NULL, currentX=NULL,   geno=NULL,  numberSNPselected=0, quiet=TRUE, gamma=NULL)
  {
    ## internal function: used by AM 
    ## smallest extBIC and BIC is best
    ## internal function: use in AM only
 
-   res_p <- emma.MLE(y=trait, X= currentX , K=MMt, Z=Zmat, llim=-100,ulim=100)
+   #res_p <- emma.MLE(y=trait, X= currentX , K=MMt, Z=Zmat, llim=-100,ulim=100)
+   #BIC <- -2 * res_p$ML + (ncol(currentX)+1) * log(length(trait))  ## fixed effects + variance component
+
+   BIC <- -2 * ML + (ncol(currentX)+1) * log(length(trait))  ## fixed effects + variance component
 
-   BIC <- -2 * res_p$ML + (ncol(currentX)+1) * log(length(trait))  ## fixed effects + variance component
   # calculate gamma
   if(is.null(gamma)){
      gamma <- 1
